/* With GCC, compile with the "-frounding-math -std=c11" options.
   The -frounding-math option may prevent GCC from doing sign-based
   optimizations, and -std=c11 at least prevents GCC from grouping
   statements to expressions in order to generate FMA instructions,
   which are not allowed here.
*/

#include <fenv.h>
#pragma STDC FENV_ACCESS ON

#ifndef DIR
#define DIR 0
#endif

#if DIR > 0     /* RU */
#define SK -
#else           /* RN and RD */
#define SK +
#endif

#ifdef S
typedef float num_t;
#define PREC 24
#define K 0x1.000aaap+0
#define L 0x1001
#else
typedef double num_t;
#define PREC 53
#define K 0x1.0000002aaaaabp+0
#define L 0x8000001
#endif

typedef struct { num_t h, l; } dword_t;

dword_t split (num_t a)
{
  num_t b, c, d, e, h;

  /* For RN (DIR == 0), do not use (+1) * a, since this cannot
     be optimized if signaling NaNs are supported. */
#if DIR == 0
  b = a;
#else
  b = (SK K) * a;
#endif
  c = L * b;
  d = b - c;
  e = c + d;
  h = SK e;
  return (dword_t) { .h = h, .l = a - h };
}
